// PascalSintAnalyzer.cpp: implementation of the CPascalCompiler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "zscpascal.h"
#include "PascalCompiler.h"
#include "VirtualMachine.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPascalCompiler::CPascalCompiler(CString str): 
CPascalLexAnalyzer(str)
{
	m_nVNivel = 0;
	
	// Code Table Init Stuff
	m_nLabelCount = 1;	
	m_Code = NULL;
}


CPascalCompiler::~CPascalCompiler()
{
	POSITION pos = m_RecordList.GetHeadPosition();
	while (pos != NULL)
	{
		Record * record = m_RecordList.GetNext(pos);
		delete record;
	}
	delete m_Code;
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 6/8/99 1:22:26 PM
// Function name	: CPascalCompiler::ProgramSursa
// Description	    : The main sintactic thing
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::ProgramSursa()
{
	// Initialize the level
	// The leveling system will be American style the base level will be 1

	m_nVNivel = 0;

	IncreaseLevel();
	if (NextToken() != TT_KW_PROGRAM)
		throw error(SET_EXPECTED,CString("\"program\""));
	
	if (NextToken() != TT_WORD)
		throw error(SET_EXPECTED, CString("identifier"));
	
	// insert a symbol for the programm

	Symbol *simb = new Symbol;
	simb->m_sName = GetStrValue();
	simb->m_nNivel = m_nVNivel;
	InsertSymbol(simb);

	if (NextToken() != ';')
		throw error(SET_EXPECTED, CString(";"));

	Bloc(simb);
	m_EntryPoint = simb->m_nAdrStart ;

	if (NextToken() != '.')
		throw error(SET_EXPECTED, CString("."));
	DecreaseLevel();
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/25/99 6:18:55 PM
// Function name	: CPascalCompiler::Bloc
// Description	    : This function processes a program block
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::Bloc(Symbol *simbb)
{
	int val, varSize = 0;
	while  (1)
	{
		if ((val = NextToken()) == TT_KW_CONST)
			ListaDeclConst();
		else
			if (val == TT_KW_VAR)
				ListaDeclVar(varSize);
			else 
				if (val == TT_KW_PROCEDURE)
				{
					IncreaseLevel();
					ParamList lp;
					CString nume;
					AntetProc(lp,nume);

					

					Symbol *simb = new Symbol;
					simb->m_sName = nume;
					simb->m_nClass = CT_PROCEDURE;
					simb->m_nNivel = m_nVNivel;
					simb->m_nAdrStart = -1;
					simb->m_ListaPar .AddTail(&lp);
					simb->m_nNrPar = lp.GetCount ();
					InsertSymbol(simb);
					
					m_pCurentBlock = simb;
					Bloc(simb);
					m_pCurentBlock = simbb;

					Symbol *simbaux = new Symbol;
					*simbaux = *simb;
					if (NextToken() != ';')
						throw error(SET_EXPECTED, CString(";"));
					DecreaseLevel();
					InsertSymbol(simbaux);

				}
				else 
					if (val == TT_KW_FUNCTION)
					{
						IncreaseLevel();
						int type;
						ParamList lp;
						CString nume;
						AntetFunc(lp,type,nume);

						Symbol *simb = new Symbol;
						simb->m_sName = nume;
						simb->m_nClass = CT_FUNCTION;
						simb->m_nNivel = m_nVNivel;
						simb->m_nAdrStart = -1;
						simb->m_nType = type;

						simb->m_ListaPar .AddTail(&lp);
						simb->m_nNrPar = lp.GetCount ();
						InsertSymbol(simb);
						m_pCurentBlock = simb;
						Bloc(simb);
						m_pCurentBlock = simbb;
						Symbol *simbaux = new Symbol;
						*simbaux = *simb;
						if (NextToken() != ';')
							throw error(SET_EXPECTED, CString(";"));
						DecreaseLevel();
						InsertSymbol(simbaux);
					}
					else
					{
						PushBack();
						// Block code beginning generation
						AddComment(simbb->m_sName  + " code beginning");
						simbb->m_nAdrStart = m_Code->GetCurCodePos();
						// now some address relocation

						POSITION pos = m_AdrList.GetHeadPosition();
						while (pos != NULL)
						{
							POSITION prev = pos;
							AddressPlace place = m_AdrList.GetNext(pos);
							if (simbb->m_sName == place.m_sSimb)
							{
								m_Code->SetAddress(place.m_nPosition,simbb->m_nAdrStart);
								m_AdrList.RemoveAt(prev);
							}
						}

						GenerateCode(INSTR_ENTER,(BYTE)m_nVNivel,(REGISTER) varSize);
						InstrCompusa();
						GenerateCode(INSTR_LEAVE);
						GenerateCode(INSTR_RET);
						break;
					}
	}
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 3:42:39 PM
// Function name	: CPascalCompiler::ListaDeclConst
// Description	    : The constant declaration list
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::ListaDeclConst()
{
	while (1)
	{
		DeclConst();
		if (NextToken() != ';')
			throw error(SET_EXPECTED, CString(";"));
		if (NextToken() != TT_WORD)
		{
			PushBack();
			break;
		}
		PushBack();
	}
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 3:43:19 PM
// Function name	: CPascalCompiler::DeclConst
// Description	    : a constant declaration
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::DeclConst()
{
	Symbol *simb = new Symbol;	// The symbol
	
	if (NextToken() != TT_WORD)
		throw error(SET_EXPECTED, CString("identifier"));
	simb->m_sName = GetStrValue();
	if (NextToken() != '=')
		throw error(SET_EXPECTED, CString("="));
	
	ExprInfo ct;
	ExprStatica(ct);

	simb->m_nClass = CT_CONST;
	simb->m_nNivel = m_nVNivel;
	simb->m_nType = ct.m_nAtribTip ;
	switch (simb->m_nType)
	{
	case ET_INTEGER:
		CV_INTEGER(simb->m_Val) = ct.m_nAttribValue ;
		break;
	case ET_REAL:
		CV_REAL(simb->m_Val) = ct.m_dAttribValue ;
		break;
	case ET_CHAR:
		CV_CHAR(simb->m_Val) = ct.m_cAttribValue ;
		break;
	}
	InsertSymbol (simb);
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 3:43:52 PM
// Function name	: CPascalCompiler::ListaDeclVar
// Description	    : The variable Declaration list
// Return type		: void 
// Argument         : int &varSize - the size of the variables
///////////////////////////////////////////////////////////////

void CPascalCompiler::ListaDeclVar(int &varSize)
{
	while (1)
	{
		DeclVar(varSize);
		if (NextToken() != ';')
			throw error(SET_EXPECTED, CString(";"));
		if (NextToken() != TT_WORD)
		{
			PushBack();
			break;
		}
		PushBack();
	}
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:12:32 PM
// Function name	: CPascalCompiler::AntetProc
// Description	    : Procedure declaration
// Return type		: void 
// Argument         : ParamList &listaParam - parameter list
// Argument         : CString &nume -  the function name
///////////////////////////////////////////////////////////////

void CPascalCompiler::AntetProc(ParamList &listaParam, CString &nume)
{
	if (NextToken()!= TT_WORD)
		throw error(SET_EXPECTED, CString("identifier"));	
	nume = GetStrValue();
	if (NextToken()== ';')
		return;
	PushBack();
	Parametri(listaParam, nume);
	if (NextToken()!= ';')
		throw error(SET_EXPECTED, CString(";"));
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:16:10 PM
// Function name	: CPascalCompiler::AntetFunc
// Description	    : Function declaration
// Return type		: void 
// Argument         : ParamList &listaParam - parameters list
// Argument         : int &type - the returning value type
// Argument         : CString &nume - the name of the function
///////////////////////////////////////////////////////////////

void CPascalCompiler::AntetFunc(ParamList &listaParam, int &type, CString &nume)
{
	if (NextToken()!= TT_WORD)
		throw error(SET_EXPECTED, CString("identifier"));;
	nume = GetStrValue();
	if (NextToken()!= ':')
	{
		PushBack();
		Parametri(listaParam, nume);
		if (NextToken()!= ':')
			throw error(SET_EXPECTED, CString(":"));
	}
	TypeInfo info;
	TipSimplu(info);
	if (NextToken()!= ';')
		throw error(SET_EXPECTED, CString(";"));
	type = info.m_nType ;
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:24:05 PM
// Function name	: CPascalCompiler::Tip
// Description	    : Type parsing
// Return type		: void 
// Argument         : TypeInfo &info - this structure is filled by the function
///////////////////////////////////////////////////////////////

void CPascalCompiler::Tip(TypeInfo &info)
{
	int val = NextToken();
	if (val == TT_KW_INTEGER
		|| val == TT_KW_REAL
		|| val == TT_KW_CHAR)
	{
		PushBack();
		info.m_nClass = CT_VAR_SIMP;
		TipSimplu(info);
	}
	if (val == TT_KW_ARRAY)
	{
		PushBack();
		info.m_nClass = CT_VAR_ARRAY;
		TipTablou(info);
	}
	if (val == TT_KW_RECORD)
	{
		PushBack();
		info.m_nClass = CT_VAR_RECORD;
		TipRecord(info);
	}
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:31:41 PM
// Function name	: CPascalCompiler::TipSimplu
// Description	    : Simple type parsing
// Return type		: void 
// Argument         : TypeInfo& info - the info is returned by function
///////////////////////////////////////////////////////////////

void CPascalCompiler::TipSimplu(TypeInfo& info)
{
	int val = NextToken();
	if (val == TT_KW_INTEGER)
	{
		info.m_nType = ET_INTEGER;
		info.m_nSize = SC_INT;
	}
	else 
		if (val == TT_KW_REAL)
		{
			info.m_nType = ET_REAL;
			info.m_nSize = SC_REAL;
		}
		else
			if (val == TT_KW_CHAR)
			{
				info.m_nType = ET_CHAR;
				info.m_nSize = SC_CHAR;
			}
			else 
				throw error(SET_EXPECTED, CString("Basic Type"));;
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:33:15 PM
// Function name	: CPascalCompiler::Parametri
// Description	    : Parameter parsing
// Return type		: void 
// Argument         : ParamList &listaParam - the returned parameter list
// Argument         : CString &function - da name
///////////////////////////////////////////////////////////////

void CPascalCompiler::Parametri(ParamList &listaParam,CString &function)
{
	if (NextToken()!= '(')
		throw error(SET_EXPECTED, CString("("));;
	while (1)
	{
		DeclParam(listaParam, function);
		if (NextToken()!= ';')
		{
			PushBack();
			break;
		}
	}
	if (NextToken()!= ')')
		throw error(SET_EXPECTED, CString(")"));;
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:34:02 PM
// Function name	: CPascalCompiler::DeclParam
// Description	    : 
// Return type		: void 
// Argument         : ParamList &listaParam
// Argument         : CString &function
///////////////////////////////////////////////////////////////

void CPascalCompiler::DeclParam(ParamList &listaParam, CString &function)
{
	int transmisie = LV_VALUE;
	StringList list;
	if (NextToken() == TT_KW_VAR)
	{
		transmisie = LV_ADDRESS;
		NextToken();
	}
	PushBack();
	while (1)
	{
		if (NextToken() != TT_WORD)
			throw error(SET_EXPECTED, CString("identifier"));;
		list.AddTail(GetStrValue());
		if (NextToken()!=',')
		{
			PushBack();
			break;
		}
	}
	
	if (NextToken() != ':')
			throw error(SET_EXPECTED, CString(":"));

	TypeInfo info;
	TipSimplu(info);

	POSITION pos = list.GetHeadPosition();
	while (pos != NULL)
	{
		CString numepar = list.GetNext(pos);
		Symbol *simb = new Symbol;
		simb->m_sName = numepar;
		if (transmisie == LV_VALUE)
			simb->m_nClass = CT_PARAM_VAL;
		else
			simb->m_nClass = CT_PARAM_ADR;
		simb->m_nType = info.m_nType ;
		simb->m_nNivel = m_nVNivel;
		Param par;
		par.m_nTransmisie =transmisie;
		par.m_nType = info.m_nType ;
		listaParam.AddTail (par);
		simb->m_nAdrel = ((int)listaParam.GetCount());
		simb->m_sRelated = function;
		InsertSymbol(simb);
	}

}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:34:11 PM
// Function name	: CPascalCompiler::InstrCompusa
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::InstrCompusa()
{
	if (NextToken()!= TT_KW_BEGIN)
		throw error(SET_EXPECTED, CString("begin"));;
	while (1)
	{
		Instr();
		if (NextToken()==TT_KW_END)
		{
			PushBack();
			break;
		}
		PushBack();
		if (NextToken()==';')
			if (NextToken()==TT_KW_END)
			{
				PushBack();
				break;
			}
			else PushBack();
		else throw error(SET_EXPECTED, CString("end or ;"));
	}
	if (NextToken()!=TT_KW_END)
		throw error(SET_EXPECTED, CString("end"));
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:34:20 PM
// Function name	: CPascalCompiler::ExprStatica
// Description	    : 
// Return type		: void 
// Argument         : ExprInfo &ct
///////////////////////////////////////////////////////////////

void CPascalCompiler::ExprStatica(ExprInfo &ct)
{
	
	ExprInfo cres;
	REAL rResult = 0;	
	CHAR cResult = 0;
	INTEGER	nResult = 0;
	int type = 0;

	int c = '+';
	while (1)
	{
		TermenStatic(cres);

		switch (type)	// tipul termenului precedent
		{
		case ET_REAL:
			if (cres.m_nAtribTip  !=ET_REAL)
				throw error(SET_INVALID_OP_TYPES,CString("real type expected"));
			switch (c)
			{
			case '+':
				rResult += cres.m_dAttribValue ;
				break;
			case '-':
				rResult -= cres.m_dAttribValue ;
				break;
			}

		case ET_INTEGER:
			if (cres.m_nAtribTip  !=ET_INTEGER)
				throw error(SET_INVALID_OP_TYPES,CString("integer type expected"));
			switch (c)
			{
			case '+':
				nResult += cres.m_nAttribValue ;
				break;
			case '-':
				nResult -= cres.m_nAttribValue ;
				break;
			}
			break;
		case ET_CHAR:
			if (cres.m_nAtribTip  !=type)
				throw error(SET_INVALID_OP_TYPES,CString("Char type expected"));
			switch (c)
			{
			case '+':
				cResult += cres.m_cAttribValue;
				break;
			case '-':
				throw error(SET_INVALID_OP,CString("Can'tsubstract two strings"));
				break;
			}
			break;
		default:
			cResult = cres.m_cAttribValue ;
			rResult = cres.m_dAttribValue ;
			nResult = cres.m_nAttribValue ;
			type = cres.m_nAtribTip;
		}
		int val = NextToken();
		if (val!='+' && val != '-')
		{
			PushBack();
			break;
		}
		c = val;
	}
	ct.m_dAttribValue  = rResult;
	ct.m_cAttribValue  = cResult;
	ct.m_nAttribValue  = nResult;
	ct.m_nAtribTip  = type;
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:34:30 PM
// Function name	: CPascalCompiler::DeclVar
// Description	    : 
// Return type		: void 
// Argument         : int &varSize
///////////////////////////////////////////////////////////////

void CPascalCompiler::DeclVar(int &varSize)
{
	TypeInfo info;
	while (1)
	{
		if (NextToken() != TT_WORD)
			throw error(SET_EXPECTED, CString("identifier"));
		info.m_VarList .AddTail (GetStrValue());	// insert var name to the list
		if (NextToken()!=',')
		{
			PushBack();
			break;
		}
	}
	if (NextToken()!=':')
		throw error(SET_EXPECTED, CString(":"));
	Tip(info);
	// insert tyhe symbols in the symbol table
	POSITION pos = info.m_VarList .GetHeadPosition ();
	while (pos != NULL)
	{
		CString name = info.m_VarList .GetNext(pos);
		Symbol *simb =  new Symbol;
		simb->m_nNivel = m_nVNivel;
		simb->m_nClass = info.m_nClass ;
		simb->m_nType = info.m_nType ;
		simb->m_nIndMin = info.m_nIndMin ;
		simb->m_nIndMax = info.m_nIndMax ;
		simb->m_sName = name;
		simb->m_nAdrel = varSize ;
		varSize += info.m_nSize;
		InsertSymbol(simb);
	}
	// hey am I cool or what?
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:34:39 PM
// Function name	: CPascalCompiler::TipTablou
// Description	    : 
// Return type		: void 
// Argument         : TypeInfo& info
///////////////////////////////////////////////////////////////

void CPascalCompiler::TipTablou(TypeInfo& info)
{
	if (NextToken() != TT_KW_ARRAY)
		throw error(SET_EXPECTED, CString("array"));
	if (NextToken() != '[')
		throw error(SET_EXPECTED, CString("["));

	if (NextToken()	!= TT_INTEGER)	// trebuie modificat pentru numar zecimal
		throw error(SET_EXPECTED, CString("number"));
	info.m_nIndMin = (int) GetNumValue();
	if (NextToken() != TT_DOTDOT)
		throw error(SET_EXPECTED, CString(".."));
	
	if (NextToken() != TT_INTEGER)	// trebuie modificat pentru numar zecimal
		throw error(SET_EXPECTED, CString("number"));
	info.m_nIndMax = (int) GetNumValue();
	// verifica daca indicii sunt corecti (indmin<indmax)
	if (info.m_nIndMin>=info.m_nIndMax)
		throw error(SET_GENERAL,CString("The Ind MAX must be greater than Ind MIN"));
	if (NextToken() != ']')
		throw error(SET_EXPECTED, CString("]"));
	if (NextToken() != TT_KW_OF)
		throw error(SET_EXPECTED, CString("of"));
	TipSimplu(info);
	info.m_nSize *= info.m_nIndMax - info.m_nIndMin + 1;
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:34:48 PM
// Function name	: CPascalCompiler::TipRecord
// Description	    : 
// Return type		: void 
// Argument         : TypeInfo& info
///////////////////////////////////////////////////////////////

void CPascalCompiler::TipRecord(TypeInfo& info)
{
	info.m_nSize  = 0;
	if (NextToken() != TT_KW_RECORD)
		throw error(SET_EXPECTED, CString("record"));;
	while (1)
	{
		DeclCamp(info);
		int val = NextToken();
		if (val != ';')
		{
			PushBack();
			break;
		}
	}
	if (NextToken() != TT_KW_END)
		throw error(SET_EXPECTED, CString("end"));
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:34:56 PM
// Function name	: CPascalCompiler::DeclCamp
// Description	    : 
// Return type		: void 
// Argument         : TypeInfo&info
///////////////////////////////////////////////////////////////

void CPascalCompiler::DeclCamp(TypeInfo&info)
{
	StringList list;
	while (1)
	{
		if (NextToken()!= TT_WORD)
			throw error(SET_EXPECTED, CString("identifier"));
		list.AddTail(GetStrValue());
		if (NextToken()!= ',')
		{
			PushBack();
			break;
		}
	}
	if (NextToken()!=':')
		throw error(SET_EXPECTED, CString(":"));
	TypeInfo secinfo;
	TipSimplu(secinfo);
	// si acum sa introducem un simbol pentru bullshit-ul de camp de record
	POSITION pos = list.GetHeadPosition ();
	while (pos!=NULL)
	{
		CString name = list.GetNext(pos);
		Symbol *simb = new Symbol;
		simb->m_sName = name;
		simb->m_nClass = CT_VAR_RECORD_FIELD;
		simb->m_nType  = secinfo.m_nType ;
		simb->m_nDeplRec = info.m_nSize ;
		info.m_nSize  += secinfo.m_nSize ;
		simb->m_ListaRec .RemoveAll();
		simb->m_ListaRec .AddTail(&info.m_VarList);	// legaturile spre variabilele care sunt de tipul record
		InsertSymbol(simb);
	}
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:35:07 PM
// Function name	: CPascalCompiler::TermenStatic
// Description	    : 
// Return type		: void 
// Argument         : ExprInfo &ct
///////////////////////////////////////////////////////////////

void CPascalCompiler::TermenStatic(ExprInfo &ct)
{
	REAL dResult = 1;
	CHAR cResult = 0;
	INTEGER nResult = 1;
	int type = 0;
	ExprInfo cres;
	int c;
	
	while (1)
	{
		FactorStatic(cres);

		switch (type)	// tipul termenului precedent
		{
		case ET_REAL:
			if (cres.m_nAtribTip  !=ET_REAL)
				throw error(SET_INVALID_OP_TYPES,CString("real type expected"));
			switch (c)
			{
			case '*':
				dResult *= cres.m_dAttribValue ;
				break;
			case '/':
				dResult /= cres.m_dAttribValue ;
				break;
			default:
				throw error(SET_INVALID_OP,CString(""));
			}

		case ET_INTEGER:
			if (cres.m_nAtribTip  !=ET_INTEGER)
				throw error(SET_INVALID_OP_TYPES,CString("integer type expected"));
			switch (c)
			{
			case '*':
				nResult *= cres.m_nAttribValue ;
				break;			
			case TT_KW_DIV:
				nResult /= cres.m_nAttribValue ;
				break;
			case TT_KW_MOD:
				nResult %= cres.m_nAttribValue ;
				break;
			default:
				throw error(SET_INVALID_OP,CString("you can't divide two integers"));
			}
			break;
		case ET_CHAR:
			throw error(SET_INVALID_OP,CString("You can't do that with char's"));
			break;
		default:
			cResult = cres.m_cAttribValue ;
			dResult = cres.m_dAttribValue ;
			nResult = cres.m_nAttribValue ;
			type = cres.m_nAtribTip ;
		}

		int val = NextToken();
		if (val!='*' && val != '/' && val!=TT_KW_MOD && val != TT_KW_DIV)
		{
			PushBack();
			break;
		}
		c = val;
	}
	ct.m_dAttribValue  = dResult;
	ct.m_cAttribValue  = cResult;
	ct.m_nAttribValue  = nResult;
	ct.m_nAtribTip  = type;
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:35:18 PM
// Function name	: CPascalCompiler::FactorStatic
// Description	    : 
// Return type		: void 
// Argument         : ExprInfo &ct
///////////////////////////////////////////////////////////////

void CPascalCompiler::FactorStatic(ExprInfo &ct)
{
	int val = NextToken();
	if (val == TT_WORD)
	{
		// search for another constant
		Symbol *simb = NULL;
		if (!RetreaveSymbol(GetStrValue(),simb))
			throw error(SET_GENERAL,CString("Constant expected"));
		if (simb->m_nClass!=CT_CONST)
			throw error(SET_GENERAL,CString("Constant expected"));
		ct.m_nAtribTip = simb->m_nType;
		switch (simb->m_nType)
		{
		case ET_INTEGER:
			ct.m_nAttribValue = CV_INTEGER(simb->m_Val) ;
			break;
		case ET_REAL:
			ct.m_dAttribValue = CV_REAL(simb->m_Val) ;
			break;
		case ET_CHAR:
			ct.m_cAttribValue = CV_CHAR(simb->m_Val) ;
			break;
		}

		return;	// this can be only a constant
	}
	if (val == TT_INTEGER)
	{
		ct.m_nAttribValue  = (INTEGER) GetNumValue ();
		ct.m_nAtribTip  = ET_INTEGER;
		return;
	}
	if (val == TT_REAL)
	{
		ct.m_dAttribValue  = (REAL) GetNumValue ();
		ct.m_nAtribTip  = ET_REAL;
		return;
	}
	if (val == TT_CHAR)
	{
		ct.m_cAttribValue  = GetStrValue ()[0];
		ct.m_nAtribTip  = ET_CHAR;
		return;
	}
	if (val == '(')
	{
		ExprStatica(ct);
		if (NextToken()!=')')
			throw error(SET_EXPECTED, CString(")"));
		return;
	}
	throw error(SET_EXPECTED, CString("Identifier or number or constant"));
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:35:28 PM
// Function name	: CPascalCompiler::Instr
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::Instr()
{
	int val = NextToken();
	switch (val)
	{
	case TT_KW_WHILE:
		{
			PushBack();
			InstrWhile();
			break;
		}
	case TT_KW_REPEAT:
		{
			PushBack();
			InstrRepeat();
			break;
		}
	case TT_KW_FOR:
		{
			PushBack();
			InstrFor();
			break;
		}
	case TT_KW_CASE:
		{
			PushBack();
			InstrCase();
			break;
		}
	case TT_KW_BEGIN:
		{
			PushBack();
			InstrCompusa();
			break;
		}
	case TT_KW_PRINT:
		{
			PushBack();
			InstrPrint();
			break;
		}
	case TT_KW_READ:
		{
			PushBack();
			InstrRead();
			break;
		}
	case TT_KW_IF:
		{
			PushBack();
			InstrIf();
			break;
		}
	case TT_WORD:
			if (IsProc(GetStrValue()))
			{
				PushBack();
				ApelProcedura();
			}
			else
			{
				PushBack();
				Atribuire();
			}
			break;
	default:		 
			throw error(SET_EXPECTED, CString("Instruction"));
	}
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:35:38 PM
// Function name	: CPascalCompiler::InsertSymbol
// Description	    : 
// Return type		: void 
// Argument         : Symbol *s
///////////////////////////////////////////////////////////////

void CPascalCompiler::InsertSymbol(Symbol *s)
{
	Symbol *simb=NULL;
	if (m_SymbTableCollection.RetreaveSymbolCL(s->m_sName,simb))
	{
		throw error(SET_GENERAL,CString(s->m_sName)+CString(" item is already declared "));
	}
	m_SymbTableCollection.InsertSymbol (s);
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/28/99 6:37:10 PM
// Function name	: CPascalCompiler::IsProc
// Description	    : Check if the parameter is a procedure
// Return type		: BOOL 
// Argument         : CString &str
///////////////////////////////////////////////////////////////

BOOL CPascalCompiler::IsProc(CString &str)
{
	Symbol *simb=NULL;
	if (!RetreaveSymbol(str,simb))
		return FALSE;
	return (simb->m_nClass == CT_PROCEDURE);
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:35:50 PM
// Function name	: CPascalCompiler::Atribuire
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::Atribuire()
{
	// Variabila
	ExprInfo t1;
	ExprInfo t2;
	VariabilaAdr(t1);
	if (NextToken()!=TT_IS)
		throw error(SET_EXPECTED, CString(":="));
	Expr(t2);
	if (t1.m_nAtribLValue == LV_VALUE)
		throw error(SET_GENERAL, CString("Invalid variable"));
	if (t1.m_nAtribTip != t2.m_nAtribTip )
	{
		if (!(t1.m_nAtribTip == ET_REAL && t2.m_nAtribTip !=ET_CHAR))
			throw error(SET_GENERAL, CString("Trying to assign between incompatible types"));
	}
	GenerateCode(INSTR_STO);
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/27/99 7:13:38 PM
// Function name	: CPascalCompiler::Variabila
// Description	    : This proceses a variable for evaluation purposes, value is put on the stack
// Return type		: void 
// Argument         : ExprInfo &type
///////////////////////////////////////////////////////////////

void CPascalCompiler::Variabila(ExprInfo &type)
{
	if (NextToken()!=TT_WORD)
		throw error(SET_EXPECTED, CString("identifier"));
	CString numevar = GetStrValue();
	int val	= NextToken();
	if (val=='[')
	{
		// Matrix
		Expr(type);	// Puts the index of the matrix on the stack

		if (NextToken()!=']')
			throw error(SET_EXPECTED, CString("]"));
		if (type.m_nAtribTip != ET_INTEGER)
			throw error(SET_GENERAL,CString("Table must have integer index"));
		Symbol *simb = NULL;
		if (!RetreaveSymbol(numevar,simb))
			throw error(SET_GENERAL,CString("Undeclared variable: ")+CString(numevar));
		if (simb->m_nClass!=CT_VAR_ARRAY)
				throw error(SET_GENERAL,CString("Invalid variable usage: ")+CString(numevar));
		type.m_nAtribLValue = LV_ADDRESS;
		type.m_nAtribTip = simb->m_nType ;
		// Generate code
		// put the variable on the stack
		// the index is on the stack Lets move it to the RX register
		GenerateCode(INSTR_MVRX);
		// Now we can try to push the value on the stack
		// let us check the level of the value with the current level
		
		if (m_nVNivel == simb->m_nNivel )
		{
			// No Frame change needed
			GenerateCode(INSTR_LODIX,(OFFSET) (simb->m_nAdrel+m_nVNivel+1));
		}
		else
		{
			// Frame Change needed
			GenerateCode(INSTR_CHGSFP,(BYTE)simb->m_nNivel);
			GenerateCode(INSTR_LODIX,(OFFSET)(simb->m_nAdrel+simb->m_nNivel+1));
			// Remake the frame
			GenerateCode(INSTR_RSTSFP);
		}


		return;
	}
	else
		if (val=='.')
		{
			if (NextToken()!=TT_WORD)
				throw error(SET_EXPECTED, CString("identifier"));
			CString numecamp = GetStrValue();
			
			Symbol *simb1 = NULL,*simb2 = NULL;
			if (!RetreaveSymbol(numevar,simb1))
				throw error(SET_GENERAL,CString("Undeclared variable: ")+CString(numevar));
			if (simb1->m_nClass!=CT_VAR_RECORD)
				throw error(SET_GENERAL,CString("Invalid variable usage: ")+CString(numevar));

			if (!RetreaveSymbol(numecamp,simb2))
				throw error(SET_GENERAL,CString("Invalid record field: ")+CString(numecamp));
			if (simb2->m_nClass!=CT_VAR_RECORD_FIELD)
				throw error(SET_GENERAL,CString("Invalid record field: ")+CString(numecamp));
			if (simb2->m_ListaRec.Find(numevar)==NULL)
				throw error(SET_GENERAL,CString("Invalid record field: ")+CString(numecamp));

			type.m_nAtribLValue = LV_ADDRESS;
			type.m_nAtribTip = simb2->m_nType ;

			// Generate code for putting the shit on the stack

			if (m_nVNivel == simb1->m_nNivel )
			{
				// No Frame change needed
				GenerateCode(INSTR_LOD,(OFFSET) (simb1->m_nAdrel+simb2->m_nDeplRec +m_nVNivel+1));
			}
			else
			{
				// Frame Change needed
				GenerateCode(INSTR_CHGSFP,(BYTE)simb1->m_nNivel);
				GenerateCode(INSTR_LOD,(OFFSET)(simb1->m_nAdrel+simb1->m_nNivel + simb2->m_nDeplRec +1 ));
				// Remake the frame
				GenerateCode(INSTR_RSTSFP);
			}

			return;
		}
		else
		{
			Symbol *simb = NULL;
			if (!RetreaveSymbol(numevar,simb))
				throw error(SET_GENERAL,CString("Undeclared variable: ")+CString(numevar));
			if (simb->m_nClass!=CT_VAR_SIMP && 
				simb->m_nClass!=CT_PARAM_VAL && 
				simb->m_nClass!=CT_PARAM_ADR &&
				simb->m_nClass!=CT_CONST)
				throw error(SET_GENERAL,CString("Invalid variable : ")+CString(numevar));

			type.m_nAtribTip = simb->m_nType ;	
			if (simb->m_nClass==CT_CONST)
				{
					type.m_nAtribLValue = LV_VALUE;
					StackEntry entry;
					entry.m_nType = simb->m_nType;
					entry.m_Info = simb->m_Val ;
					GenerateCode(INSTR_LODI,entry);
				}
			
				type.m_nAtribLValue = LV_ADDRESS;
				// Generate code for normal stuff
				
				
				if (m_nVNivel != simb->m_nNivel)
					GenerateCode(INSTR_CHGSFP,(BYTE)simb->m_nNivel);

				Symbol *auxsimb = NULL;
				int np = 0;
				if (RetreaveSymbol(simb->m_sRelated,auxsimb))
					np = auxsimb->m_ListaPar.GetCount();
				switch (simb->m_nClass)
				{
				case CT_VAR_SIMP:
					GenerateCode(INSTR_LOD,(OFFSET) (simb->m_nAdrel+simb->m_nNivel+1));
					break;
				case CT_PARAM_VAL:
					GenerateCode(INSTR_LOD,(OFFSET) (simb->m_nAdrel-np -2));
					break;
				case CT_PARAM_ADR:
					GenerateCode(INSTR_LODEA,(OFFSET) (simb->m_nAdrel-np-2));
					break;

				}
				if (m_nVNivel != simb->m_nNivel)
					GenerateCode(INSTR_RSTSFP);

			PushBack();
		}
}





///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/27/99 7:15:53 PM
// Function name	: CPascalCompiler::Variabila
// Description	    : This variable stuff is used for addressing purposes
//					: at the left side of a assignement, put's the address to the stack
// Return type		: void 
// Argument         : ExprInfo &type
// Argument         : REGISTER & address
// Argument         : int level
///////////////////////////////////////////////////////////////

void CPascalCompiler::VariabilaAdr(ExprInfo &type)
{
	// set default value for relative stuff

	if (NextToken()!=TT_WORD)
		throw error(SET_EXPECTED, CString("identifier"));
	CString numevar = GetStrValue();
	int val	= NextToken();
	if (val=='[')
	{
		// Matrix
		Expr(type);	// Puts the index of the matrix on the stack

		if (NextToken()!=']')
			throw error(SET_EXPECTED, CString("]"));
		if (type.m_nAtribTip != ET_INTEGER)
			throw error(SET_GENERAL,CString("Table must have integer index"));
		Symbol *simb = NULL;
		if (!RetreaveSymbol(numevar,simb))
			throw error(SET_GENERAL,CString("Undeclared variable: ")+CString(numevar));
		if (simb->m_nClass!=CT_VAR_ARRAY)
				throw error(SET_GENERAL,CString("Invalid variable usage: ")+CString(numevar));
		type.m_nAtribLValue = LV_ADDRESS;
		type.m_nAtribTip = simb->m_nType ;

		// prepare the addres and the level
		// This is only for arrays, they ca only be 
		GenerateCode(INSTR_MVRX);	// put the index result in RX register
		if (m_nVNivel == simb->m_nNivel )
		{
			// No Frame change needed
			GenerateCode(INSTR_LODAX,(OFFSET) (simb->m_nAdrel+m_nVNivel+1));
		}
		else
		{
			// Frame Change needed
			GenerateCode(INSTR_CHGSFP,(BYTE)simb->m_nNivel);
			GenerateCode(INSTR_LODAX,(OFFSET)(simb->m_nAdrel+simb->m_nNivel+1));
			// Remake the frame
			GenerateCode(INSTR_RSTSFP);
		}


		
		return;
	}
	else
		if (val=='.')
		{
			// REcord
			if (NextToken()!=TT_WORD)
				throw error(SET_EXPECTED, CString("identifier"));
			CString numecamp = GetStrValue();
			
			Symbol *simb1 = NULL,*simb2 = NULL;
			if (!RetreaveSymbol(numevar,simb1))
				throw error(SET_GENERAL,CString("Undeclared variable: ")+CString(numevar));
			if (simb1->m_nClass!=CT_VAR_RECORD)
				throw error(SET_GENERAL,CString("Invalid variable usage: ")+CString(numevar));

			if (!RetreaveSymbol(numecamp,simb2))
				throw error(SET_GENERAL,CString("Invalid record field: ")+CString(numecamp));
			if (simb2->m_nClass!=CT_VAR_RECORD_FIELD)
				throw error(SET_GENERAL,CString("Invalid record field: ")+CString(numecamp));
			if (simb2->m_ListaRec.Find(numevar)==NULL)
				throw error(SET_GENERAL,CString("Invalid record field: ")+CString(numecamp));
			type.m_nAtribLValue = LV_ADDRESS;
			type.m_nAtribTip = simb2->m_nType ;

			// now generate bulshit 
			if (m_nVNivel == simb1->m_nNivel )
			{
				// No Frame change needed
				GenerateCode(INSTR_LODA,(OFFSET) (simb1->m_nAdrel+simb2->m_nDeplRec +m_nVNivel+1));
			}
			else
			{
				// Frame Change needed
				GenerateCode(INSTR_CHGSFP,(BYTE)simb1->m_nNivel);
				GenerateCode(INSTR_LODA,(OFFSET)(simb1->m_nAdrel+simb1->m_nNivel + simb2->m_nDeplRec +1 ));
				// Remake the frame
				GenerateCode(INSTR_RSTSFP);
			}		
			return;
		}
		else
		{
			Symbol *simb = NULL;
			if (!RetreaveSymbol(numevar,simb))
				throw error(SET_GENERAL,CString("Undeclared variable: ")+CString(numevar));
			if (simb->m_nClass!=CT_VAR_SIMP && 
				simb->m_nClass!=CT_PARAM_VAL && 
				simb->m_nClass!=CT_PARAM_ADR &&
				simb->m_nClass!=CT_FUNCTION)
				throw error(SET_GENERAL,CString("Invalid variable : ")+CString(numevar));
				type.m_nAtribLValue = LV_ADDRESS;
				type.m_nAtribTip = simb->m_nType ;

				if (m_nVNivel != simb->m_nNivel)
					GenerateCode(INSTR_CHGSFP,(BYTE)simb->m_nNivel);

				Symbol *auxsimb = NULL ;
				int np;
				if (RetreaveSymbol(simb->m_sRelated,auxsimb))
					np = auxsimb->m_ListaPar.GetCount();
				switch (simb->m_nClass)
				{
				case CT_VAR_SIMP:
					GenerateCode(INSTR_LODA,(OFFSET) (simb->m_nAdrel+simb->m_nNivel+1));
					break;
				case CT_PARAM_VAL:
					GenerateCode(INSTR_LODA,(OFFSET) (simb->m_nAdrel-np -2));
					break;
				case CT_PARAM_ADR:
					GenerateCode(INSTR_LOD,(OFFSET) (simb->m_nAdrel-np-2));
					break;
				case CT_FUNCTION:
					// the return value of the function
					if (m_pCurentBlock != simb)
						throw error(SET_GENERAL,CString("Invalid instruction !!!"));
					GenerateCode(INSTR_LODA,(OFFSET) (-(simb->m_ListaPar.GetCount() + 2)));
					break;

				}
				if (m_nVNivel != simb->m_nNivel)
					GenerateCode(INSTR_RSTSFP);

			PushBack();
		}
}









///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:36:12 PM
// Function name	: CPascalCompiler::Expr
// Description	    : 
// Return type		: void 
// Argument         : ExprInfo& type
///////////////////////////////////////////////////////////////

void CPascalCompiler::Expr(ExprInfo& type)
{
	ExprInfo et;
	int c='+';
	while (1)
	{
		Termen(type);
		switch (et.m_nAtribTip)	// tipul termenului precedent
		{
		case ET_REAL:
			if (type.m_nAtribTip !=ET_REAL && type.m_nAtribTip !=ET_INTEGER)
				throw error(SET_GENERAL,CString("Ivalid operation between incompatible Types"));
			// Generate code 
			if (c == '+')
				GenerateCode(INSTR_ADD);
			else 
				GenerateCode(INSTR_SUB);
			break;
		case ET_INTEGER:
			if (type.m_nAtribTip !=ET_REAL && type.m_nAtribTip !=ET_INTEGER)
				throw error(SET_GENERAL,CString("Ivalid operation between incompatible Types"));
			if (type.m_nAtribTip == ET_REAL)
				et.m_nAtribTip = ET_REAL;
			// Generate code
			if (c == '+')
				GenerateCode(INSTR_ADD);
			else 
				GenerateCode(INSTR_SUB);

			break;
		case ET_CHAR:
			if (type.m_nAtribTip != ET_CHAR)
				throw error(SET_GENERAL,CString("Ivalid operation between incompatible Types"));
			// Generate code
			if (c == '+')
				GenerateCode(INSTR_ADD);
			else 
				GenerateCode(INSTR_SUB);

			break;
		default:
			et.m_nAtribTip  = type.m_nAtribTip ;
			et.m_nAtribLValue = type.m_nAtribLValue ;
		}
		int val = NextToken();
		if (val!='+' && val != '-')
		{
			PushBack();
			break;
		}
		c = val;
		et.m_nAtribLValue = LV_VALUE;
	}
	
	type = et;
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:36:22 PM
// Function name	: CPascalCompiler::Termen
// Description	    : 
// Return type		: void 
// Argument         : ExprInfo& type
///////////////////////////////////////////////////////////////

void CPascalCompiler::Termen(ExprInfo& type)
{
	ExprInfo et;
	int c ='*';

	while (1)
	{
		Factor(type);
		switch (et.m_nAtribTip)	// tipul termenului precedent
		{
		case ET_REAL:
			if (type.m_nAtribTip !=ET_REAL && type.m_nAtribTip !=ET_INTEGER)
				throw error(SET_GENERAL,CString("Ivalid operation between incompatible Types"));
			if (c == TT_KW_MOD )
				throw error(SET_GENERAL,CString("Ivalid operation With real numbers"));
			
			// Generate code
			if (c == TT_KW_MOD)
				GenerateCode(INSTR_MOD);
			if (c == TT_KW_DIV)
				GenerateCode(INSTR_DIV);
			if (c == '/')
				GenerateCode(INSTR_DIV);
			if (c == '*')
				GenerateCode(INSTR_MUL);
			
			
			break;
		case ET_INTEGER:
			if (type.m_nAtribTip !=ET_REAL && type.m_nAtribTip !=ET_INTEGER)
				throw error(SET_GENERAL,CString("Ivalid operation between incompatible Types"));
			if (type.m_nAtribTip == ET_REAL)
			{
				if (c == TT_KW_MOD)
					throw error(SET_GENERAL,CString("Ivalid operation With real numbers"));
				et.m_nAtribTip = ET_REAL;
			}
			// Generate code
			if (c == TT_KW_MOD)
				GenerateCode(INSTR_MOD);
			if (c == TT_KW_DIV)
				GenerateCode(INSTR_DIV);
			if (c == '/')
				GenerateCode(INSTR_DIV);
			if (c == '*')
				GenerateCode(INSTR_MUL);

			break;
		case ET_CHAR:
			throw error(SET_GENERAL,CString("Ivalid operation with Strings"));
			break;
		default:
			et.m_nAtribTip  = type.m_nAtribTip ;
			et.m_nAtribLValue = type.m_nAtribLValue ;
		}

		int val = NextToken();
		if (val!='*' && val != '/' && val!=TT_KW_MOD && val != TT_KW_DIV)
		{
			PushBack();
			break;
		}
		et.m_nAtribLValue = LV_VALUE;
		c = val;
	}
	type = et;
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:36:32 PM
// Function name	: CPascalCompiler::Factor
// Description	    : 
// Return type		: void 
// Argument         : ExprInfo &type
///////////////////////////////////////////////////////////////

void CPascalCompiler::Factor(ExprInfo &type)
{
	StackEntry entry;
	int val = NextToken();
	if (val == '-')
	{
		entry.m_nType = SE_INTEGER;		
		CV_INTEGER(entry.m_Info) = (INTEGER)0;
		GenerateCode(INSTR_LODI,entry);	
		Factor(type);
		GenerateCode(INSTR_SUB);	
		return;
	}
	if (val == TT_INTEGER)
	{
		type.m_nAtribLValue = LV_VALUE;
		type.m_nAtribTip = ET_INTEGER;
		
		// Generate code
		entry.m_nType = SE_INTEGER;		
		CV_INTEGER(entry.m_Info) = (INTEGER)GetNumValue ();
		GenerateCode(INSTR_LODI,entry);
		return;
	}
	if (val == TT_REAL)
	{
		type.m_nAtribLValue = LV_VALUE;
		type.m_nAtribTip = ET_REAL;

		// Generate Code
		entry.m_nType = SE_REAL;		
		CV_REAL(entry.m_Info) = (REAL)GetNumValue ();
		GenerateCode(INSTR_LODI,entry);

		return;
	}
	if (val == TT_CHAR)
	{
		type.m_nAtribLValue = LV_VALUE;
		type.m_nAtribTip = ET_CHAR;

		// Generate Code
		entry.m_nType = SE_CHAR;		
		CV_CHAR(entry.m_Info) = (CHAR)(GetStrValue()[0]);
		GenerateCode(INSTR_LODI,entry);

		return;
	}
	if (val == '(')
	{
		Expr(type);
		if (NextToken()!=')')
			throw error(SET_EXPECTED, CString(")"));
		return;
	}
	if (val == TT_WORD)
		if (IsFunc(GetStrValue()))
		{
			PushBack();
			ApelFunc(type);	// Leaves result on stack
			return;
		}
		else 
		{
			PushBack();
			Variabila(type); // Leaves result on stack
			return;
		}
	
	throw error(SET_EXPECTED, CString("number or constant or identifier or function or variable "));;
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:36:42 PM
// Function name	: CPascalCompiler::IsFunc
// Description	    : 
// Return type		: BOOL 
// Argument         : CString &str
///////////////////////////////////////////////////////////////

BOOL CPascalCompiler::IsFunc(CString &str)
{
	Symbol *simb = NULL;
	if (!RetreaveSymbol(str,simb))
		return FALSE;
	return (simb->m_nClass == CT_FUNCTION);
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/28/99 3:35:29 PM
// Function name	: CPascalCompiler::ApelFunc
// Description	    : Function calling
// Return type		: void 
// Argument         : ExprInfo &type
///////////////////////////////////////////////////////////////

void CPascalCompiler::ApelFunc(ExprInfo &type)
{
	if (NextToken()!=TT_WORD)
		throw error(SET_EXPECTED, CString("identifier"));
	Symbol *simb = NULL;
	CString functie = GetStrValue();
	RetreaveSymbol(functie,simb);
	AddComment("Calling: " + GetStrValue());
	// alocate the return value on the stack
	StackEntry entry;
	entry.m_nType = simb->m_nType ;
	GenerateCode(INSTR_LODI,entry);

	POSITION pos = simb->m_ListaPar.GetHeadPosition ();
	if (NextToken()=='(')
	{
		while (1)
		{
			if (pos == NULL)
				throw error(SET_GENERAL, CString("Too many parameters"));
			Param par = simb->m_ListaPar .GetNext(pos);
			
			// This part put's the entire parameters on the stack
			if (par.m_nTransmisie == LV_ADDRESS)
				VariabilaAdr(type);
			else
				Expr(type);

			if (type.m_nAtribTip != par.m_nType )
				throw error(SET_GENERAL, CString("Invalid Parameter type"));
			if (par.m_nTransmisie == LV_ADDRESS && type.m_nAtribLValue== LV_VALUE )
				throw error(SET_GENERAL, CString("Invalid Parameter, must be a variable"));
			
	


			if (NextToken()!=',')
			{
				PushBack();
				if (pos != NULL)
					throw error(SET_GENERAL, CString("Too few parameters"));
				break;
			}
		}
		if (NextToken()!=')')
			throw error(SET_EXPECTED, CString(")"));
	}
	else
	{
		PushBack();
		if (simb->m_ListaPar.GetCount () != 0)
				throw error(SET_GENERAL, CString("Some parameters expected"));
	}
	type.m_nAtribLValue = LV_VALUE;
	type.m_nAtribTip = simb->m_nType ;
	// Generate code for the call
	if (simb->m_nAdrStart == -1)
	{
		AddressPlace place;
		place.m_nPosition = m_Code->GetCurCodePos() + VM_ICSIZE;
		place.m_sSimb = functie ;
		m_AdrList.AddHead(place);
	}
	GenerateCode(INSTR_CALL,(REGISTER)simb->m_nAdrStart, simb->m_sName);
	GenerateCode(INSTR_RED,(OFFSET)simb->m_ListaPar.GetCount());
	// THe result is leaved on the stack
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:36:53 PM
// Function name	: CPascalCompiler::InstrWhile
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::InstrWhile()
{
	CString label,label2;
	if (NextToken()!=TT_KW_WHILE)
		throw error(SET_EXPECTED, CString("while"));
	int ujmpAdr = m_Code->GetCurCodePos();
	label2.Format("ET%d", m_nLabelCount ++);
	InsertLabel(label2 + ":");
	Conditie();
	label.Format("ET%d",m_nLabelCount ++);
	int jumpAdr = m_Code->GetCurCodePos() + VM_ICSIZE;
	GenerateCode(INSTR_NFJP,0,label);
	if (NextToken()!=TT_KW_DO)
		throw error(SET_EXPECTED, CString("do"));
	Instr();
	GenerateCode(INSTR_UJP,ujmpAdr,label2);
	InsertLabel(label+":");
	m_Code->SetAddress(jumpAdr,m_Code->GetCurCodePos());
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 6/8/99 1:16:45 PM
// Function name	: CPascalCompiler::Conditie
// Description	    : Evaluare conditie
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::Conditie()
{
	int val;
	BOOL first = TRUE;
	int op = TT_KW_AND;
	while (1)
	{
		if (NextToken()!=TT_KW_NOT)
			PushBack();
		ExprLogica();
		val = NextToken();
		if (!first)
			if (op == TT_KW_OR)
				GenerateCode(INSTR_OR);
			else
				GenerateCode(INSTR_AND);

		if (val != TT_KW_AND && val != TT_KW_OR)
		{
			PushBack();
			break;
		}
		op = val;
		first = FALSE;
	}
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:37:02 PM
// Function name	: CPascalCompiler::ExprLogica
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::ExprLogica()
{
	ExprInfo type1, type2;
	Expr(type1);
	int val = NextToken();
	switch (val)
	{
	case '=':break;
	case '<':break;
	case '>':break;
	case TT_GTE:break;
	case TT_LWE:break;
	case TT_NE:break;
	default: throw error(SET_EXPECTED, CString("logical operator"));
	}
	Expr(type2);
	if (type1.m_nAtribTip == ET_CHAR && type2.m_nAtribTip != ET_CHAR)
		throw error(SET_EXPECTED, CString("same operand types"));
	if ((type1.m_nAtribTip == ET_INTEGER || type1.m_nAtribTip != ET_REAL) &&
		(type2.m_nAtribTip == ET_CHAR))
		throw error(SET_EXPECTED, CString("same operand types"));

	// Now generating code
	
	switch (val)
	{
	case '=':
		GenerateCode(INSTR_EQU);
		break;
	case '<':
		GenerateCode(INSTR_LES);
		break;
	case '>':break;
		GenerateCode(INSTR_GRT);
		break;
	case TT_GTE:
		GenerateCode(INSTR_GEQ);
		break;
	case TT_LWE:
		GenerateCode(INSTR_LEQ);
		break;
	case TT_NE:
		GenerateCode(INSTR_NEQ);
		break;
	}

}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:37:11 PM
// Function name	: CPascalCompiler::InstrRepeat
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::InstrRepeat()
{
	if (NextToken()!=TT_KW_REPEAT)
		throw error(SET_EXPECTED, CString("repeat"));
	int jumpAdr = m_Code->GetCurCodePos();
	CString label;
	label.Format("ET%d",m_nLabelCount ++);
	InsertLabel(label + ":");
	Instr();
	if (NextToken()!=TT_KW_UNTIL)
		throw error(SET_EXPECTED, CString("until"));
	Conditie();
	GenerateCode(INSTR_NFJP,(REGISTER)jumpAdr,label);
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:37:17 PM
// Function name	: CPascalCompiler::InstrIf
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::InstrIf()
{
	CString label;
	
	// unnecesary comparison, but ... :-) who cares
	if (NextToken()!=TT_KW_IF)
		throw error(SET_EXPECTED, CString("if"));

	Conditie();
	if (NextToken()!=TT_KW_THEN)
		throw error(SET_EXPECTED, CString("then"));

	// now we have the result of the condition on the stack
	int place1 = m_Code->GetCurCodePos()+VM_ICSIZE;
	label.Format("ET%d",m_nLabelCount++);
	GenerateCode(INSTR_NFJP,(REGISTER)0,label);
	Instr();
	int place2 = m_Code->GetCurCodePos()+VM_ICSIZE;
	CString label2 = label;
	label.Format("ET%d",m_nLabelCount++);
	GenerateCode(INSTR_UJP,(REGISTER)0,label);
	InsertLabel(label2+":");
	m_Code->SetAddress(place1,(REGISTER) m_Code->GetCurCodePos())	;
	if (NextToken()==TT_KW_ELSE)
	{
		Instr();
	}
	else PushBack();
	m_Code->SetAddress(place2,(REGISTER) m_Code->GetCurCodePos())	;
	InsertLabel(label + ":");
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 6/1/99 2:56:22 PM
// Function name	: CPascalCompiler::InstrFor
// Description	    : This function is is version one
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::InstrFor()
{
	BOOL ascending;
	BOOL stepspecified = FALSE;
	if (NextToken()!=TT_KW_FOR)
		throw error(SET_EXPECTED, CString("for"));
	ExprInfo type;
	// Start Counter address generation record
	POSITION countAdr = StartNewRecord();
	VariabilaAdr(type);
	StopRecord (countAdr);
	// The address of the variable is on the stack
	

	if (type.m_nAtribLValue!=LV_ADDRESS)
		throw error(SET_GENERAL, CString("Variable expected"));
	if (type.m_nAtribTip != ET_INTEGER)
		throw error(SET_GENERAL, CString("Variable of integer type expected"));
	if (NextToken()!=TT_IS)
		throw error(SET_EXPECTED, CString(":="));
	// now putting the initial value on the stack
	Expr(type);

	// Store the initial value in the counter
	GenerateCode(INSTR_STO);

	
	if (type.m_nAtribTip != ET_INTEGER)
		throw error(SET_GENERAL, CString("The For value must be of integer type"));
	int val = NextToken();
	if (val==TT_KW_TO)
	{
		// varianta To
		ascending = TRUE;
	}
	else
		if (val==TT_KW_DOWNTO)
		{
			// varianta DownTo
			ascending = FALSE;
		}
		else throw error(SET_EXPECTED, CString("to or downto"));

	// Now redirect the code generation to a temporary code storage
	Code * oldCode = NULL;
	Code * tmpCode = new Code;
	
	oldCode = Redirect(tmpCode);
	
	// Record the target value
	
	POSITION targetValue = StartNewRecord();
	Expr(type);
	StopRecord(targetValue);

	if (type.m_nAtribTip != ET_INTEGER)
	{
		delete oldCode;
		throw error(SET_GENERAL, CString("The For value must be of integer type"));
	}
	POSITION stepValue = NULL;
	
	if (NextToken() == TT_KW_STEP)
	{
		stepValue = StartNewRecord();
		Expr(type);
		StopRecord(stepValue);
		if (type.m_nAtribTip != ET_INTEGER)
		{
			delete oldCode;
			throw error(SET_GENERAL, CString("The For value must be of integer type"));
		}
		// now we have the increment on stack
		stepspecified = TRUE;
	}
	else PushBack();
	if (NextToken()!=TT_KW_DO)
	{
		delete oldCode;
		throw error(SET_EXPECTED, CString("do"));
	}
	
	// writing code
	Redirect(oldCode);

	int et1 = m_Code->GetCurCodePos();
	InsertLabel(CString("ET1:"));

	PasteRecord(countAdr);
	GenerateCode(INSTR_USTO);
	PasteRecord(targetValue);
	if (ascending)
		GenerateCode(INSTR_GRT);
	else
		GenerateCode(INSTR_LES);
	int place = m_Code->GetCurCodePos()+VM_ICSIZE;
	GenerateCode(INSTR_FJP,(REGISTER)0,CString("ET2"));

	Instr();

	PasteRecord(countAdr);
	GenerateCode(INSTR_COPY);
	GenerateCode(INSTR_USTO);
	if (stepspecified)
		PasteRecord(stepValue);
	else
	{
		StackEntry entry;
		entry.m_nType = SE_INTEGER;
		CV_INTEGER(entry.m_Info) = 1;
		GenerateCode(INSTR_LODI,entry);
	}
	if (ascending)
		GenerateCode(INSTR_ADD);
	else
		GenerateCode(INSTR_SUB);
	GenerateCode(INSTR_STO);
	GenerateCode(INSTR_UJP,(REGISTER) et1,CString("ET1"));
	m_Code->SetAddress(place,(REGISTER) m_Code->GetCurCodePos())	;
	InsertLabel (CString("ET2:"));

	RemoveRecord(countAdr);
	RemoveRecord(targetValue);
	if (stepValue)
		RemoveRecord(stepValue);
	delete tmpCode;
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:37:30 PM
// Function name	: CPascalCompiler::InstrCase
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::InstrCase()
{
	if (NextToken() != TT_KW_CASE)
		throw error(SET_EXPECTED, CString("case"));
	ExprInfo type;
	Expr(type);

	// Now the case expression is on the Sack

	if (NextToken() != TT_KW_OF)
		throw error(SET_EXPECTED, CString("of"));
	ListaAltern(type);
	if (NextToken() != TT_KW_END)
		throw error(SET_EXPECTED, CString("end"));
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:37:38 PM
// Function name	: CPascalCompiler::ListaAltern
// Description	    : 
// Return type		: void 
// Argument         : ExprInfo& Info
///////////////////////////////////////////////////////////////

void CPascalCompiler::ListaAltern(ExprInfo& Info)
{
	StackEntry entry;
	CString label, endlabel;
	endlabel.Format("ET%d",m_nLabelCount);
	CList<int,int> endAddressList;
	while (1)
	{
		CList<int,int> addressList;
		m_nLabelCount ++;
		while (1)
		{
			GenerateCode(INSTR_COPY);
			int val = NextToken();
			switch (val)
			{
			case TT_INTEGER:
				if (Info.m_nAtribTip != ET_INTEGER)
					throw error(SET_EXPECTED, CString("integer"));
				
				entry.m_nType = SE_INTEGER;
				CV_INTEGER(entry.m_Info) = (INTEGER) GetNumValue();
				GenerateCode(INSTR_LODI,entry);
				break;
			case TT_REAL:
				if (Info.m_nAtribTip != ET_REAL)
					throw error(SET_EXPECTED, CString("real"));
				
				entry.m_nType = SE_REAL;
				CV_REAL(entry.m_Info) = (REAL) GetNumValue();
				GenerateCode(INSTR_LODI,entry);
				break;
			case TT_CHAR:
				if (Info.m_nAtribTip != ET_CHAR)
					throw error(SET_EXPECTED, CString("char"));
	
				entry.m_nType = SE_CHAR;
				CV_CHAR(entry.m_Info) = (CHAR) GetNumValue();
				GenerateCode(INSTR_LODI,entry);
				
				break;
			default :
				throw error(SET_EXPECTED, CString("Constant"));

			}
			GenerateCode(INSTR_EQU);
			addressList.AddHead(m_Code->GetCurCodePos() + VM_ICSIZE);
			label.Format("ET%d",m_nLabelCount);
			GenerateCode(INSTR_FJP,0,label);
			if (NextToken()!=',')
			{
				PushBack();
				break;
			}
		}


		if (NextToken()!=':')
			throw error(SET_EXPECTED, CString(":"));

		int where = m_Code->GetCurCodePos() + VM_ICSIZE;
		CString tmp = label + ":";
		label.Format("ET%d",++m_nLabelCount);
		GenerateCode(INSTR_UJP,0,label);
		
		InsertLabel(tmp);
		// now setting the jumping stuff addresses
		POSITION pos = addressList.GetHeadPosition();
		while (pos != NULL)
		{
			int place = addressList.GetNext(pos);
			m_Code->SetAddress(place,m_Code->GetCurCodePos());
		}
		addressList.RemoveAll();

		Instr();
		// jump to end
		endAddressList.AddHead(m_Code->GetCurCodePos() + VM_ICSIZE);
		GenerateCode(INSTR_UJP,0,endlabel);

		InsertLabel(label + ":");
		m_Code->SetAddress(where,m_Code->GetCurCodePos());
		if (NextToken()!=';')
			{
				PushBack();
				break;
			}
	}
	if (NextToken() == TT_KW_OTHERWISE)
	{
		Instr();
	}
	else
		PushBack();
		
	// set the jump to end addresses
	POSITION pos = endAddressList.GetHeadPosition();
	while (pos != NULL)
	{
		int place = endAddressList.GetNext(pos);
		m_Code->SetAddress(place,m_Code->GetCurCodePos());
	}	
	InsertLabel(endlabel + ":");

	GenerateCode(INSTR_RED,(OFFSET) 1);	// clear the value from stack
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:37:49 PM
// Function name	: CPascalCompiler::ApelProcedura
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::ApelProcedura()
{
	ExprInfo type;
	if (NextToken()!= TT_WORD)
		throw error(SET_EXPECTED, CString("identifier"));
	Symbol *simb = NULL;
	CString procedure = GetStrValue();
	RetreaveSymbol(procedure,simb);
	
	AddComment("Calling: " + GetStrValue());
	
	POSITION pos = simb->m_ListaPar.GetHeadPosition ();
	if (NextToken()=='(')
	{
		while (1)
		{
			if (pos == NULL)
				throw error(SET_GENERAL, CString("Too many parameters"));
			Param par = simb->m_ListaPar .GetNext(pos);

			// This part put's the entire parameters on the stack
			if (par.m_nTransmisie == LV_ADDRESS)
				VariabilaAdr(type);
			else
				Expr(type);
			if (type.m_nAtribTip != par.m_nType )
				throw error(SET_GENERAL, CString("Invalid Parameter type"));
			if (par.m_nTransmisie == LV_ADDRESS && type.m_nAtribLValue== LV_VALUE )
				throw error(SET_GENERAL, CString("Invalid Parameter, must be a variable"));
			if (NextToken()!=',')
			{
				PushBack();
				if (pos != NULL)
					throw error(SET_GENERAL, CString("Too few parameters"));
				break;
			}
		}
		if (NextToken()!=')')
			throw error(SET_EXPECTED, CString(")"));
	}
	else
	{
		PushBack();
		if (simb->m_ListaPar.GetCount () != 0)
			throw error(SET_GENERAL, CString("Some parameters expected"));
	}
	// Generate code for the call
	if (simb->m_nAdrStart == -1)
	{
		AddressPlace place;
		place.m_nPosition = m_Code->GetCurCodePos() + VM_ICSIZE;
		place.m_sSimb = procedure;
		m_AdrList.AddHead(place);
	}
	GenerateCode(INSTR_CALL,(REGISTER)simb->m_nAdrStart, simb->m_sName);
	GenerateCode(INSTR_RED,(OFFSET)simb->m_ListaPar.GetCount());
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:37:58 PM
// Function name	: CPascalCompiler::InstrPrint
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::InstrPrint()
{
	if (NextToken()!=TT_KW_PRINT)
		throw error(SET_EXPECTED, CString("print"));
	if (NextToken()!='(')
		throw error(SET_EXPECTED, CString("("));
	while (1)
	{
		if (NextToken()!=TT_STRING)
		{
			PushBack();
			ExprInfo type;
			Expr(type);
			GenerateCode(INSTR_OUTP);
		}
		else
		{
			// If string
			// generate string printing code
			CString str = GetStrValue();
			for (int i =0 ; i<str.GetLength(); i++)
			{
				StackEntry entry;
				entry.m_nType = SE_CHAR;
				CV_CHAR(entry.m_Info) = str[i];
				GenerateCode(INSTR_LODI,entry);
				GenerateCode(INSTR_OUTP);
			}
		}
		if (NextToken()!=',')
		{
			PushBack();
			break;
		}
	}
	if (NextToken()!=')')
		throw error(SET_EXPECTED, CString(")"));
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:38:06 PM
// Function name	: CPascalCompiler::InstrRead
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::InstrRead()
{
	if (NextToken()!=TT_KW_READ)
		throw error(SET_EXPECTED, CString("read"));
	if (NextToken()!='(')
		throw error(SET_EXPECTED, CString("("));
	while (1)
	{
		ExprInfo type;
		VariabilaAdr(type);
		GenerateCode(INSTR_INP,(BYTE) type.m_nAtribTip);
		GenerateCode(INSTR_STO);

		if (NextToken()!=',')
		{
			PushBack();
			break;
		}
	}
	if (NextToken()!=')')
		throw error(SET_EXPECTED, CString(")"));
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:38:14 PM
// Function name	: error::GetErrStr
// Description	    : 
// Return type		: CString 
///////////////////////////////////////////////////////////////

CString error::GetErrStr()
{
	CString estr;
	switch (m_nType)
	{
	case SET_GENERAL: estr.Format("Syntax ERROR, %s",m_sData);break;
	case SET_EXPECTED: estr.Format("%s expected",m_sData);break;
	case SET_INVALID_OP_TYPES: estr.Format("Invalid operation between different types, %s",m_sData);break;
	case SET_INVALID_OP: estr.Format("Invalid operation , %s",m_sData);break;
	default: estr = _T("Syntax ERROR");
	}
	return estr;
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:38:22 PM
// Function name	: CPascalCompiler::GetErrStr
// Description	    : 
// Return type		: CString 
// Argument         : error &e
///////////////////////////////////////////////////////////////

CString CPascalCompiler::GetErrStr(error &e)
{
	CString str;
	str.Format("%s  at : %d", e.GetErrStr(), LineNo ());	
	return str;
}






///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:38:30 PM
// Function name	: SymbolTableCollection::SymbolTableCollection
// Description	    : 
// Return type		: 
///////////////////////////////////////////////////////////////

SymbolTableCollection::SymbolTableCollection()
{
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:38:37 PM
// Function name	: SymbolTableCollection::~SymbolTableCollection
// Description	    : 
// Return type		: 
///////////////////////////////////////////////////////////////

SymbolTableCollection::~SymbolTableCollection()
{
	POSITION pos = GetHeadPosition();
	while (pos!=NULL)
	{
		SymbolTable *table = GetNext(pos);
		delete table;
	}
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:38:44 PM
// Function name	: SymbolTableCollection::IncreaseLevel
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void SymbolTableCollection::IncreaseLevel()
{
	SymbolTable *ntable = new SymbolTable();
	ntable->InitHashTable( MAX_SYMB );
	AddHead(ntable);
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:38:51 PM
// Function name	: SymbolTableCollection::DecreaseLevel
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void SymbolTableCollection::DecreaseLevel()
{
	SymbolTable *ntable = GetHead();
	delete ntable;
	RemoveHead();
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:38:57 PM
// Function name	: SymbolTableCollection::InsertSymbol
// Description	    : 
// Return type		: void 
// Argument         : Symbol *s
///////////////////////////////////////////////////////////////

void SymbolTableCollection::InsertSymbol(Symbol *s)
{
	SymbolTable *ntable = GetHead();
	(*ntable)[s->m_sName] = s;
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:39:04 PM
// Function name	: SymbolTableCollection::RetreaveSymbolCL
// Description	    : 
// Return type		: BOOL 
// Argument         : CString& name
// Argument         : Symbol *&simb
///////////////////////////////////////////////////////////////

BOOL SymbolTableCollection::RetreaveSymbolCL(CString& name, Symbol *&simb)
{
	SymbolTable *ntable = GetHead();
	return ntable->Lookup(name,simb);
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:39:12 PM
// Function name	: CPascalCompiler::RetreaveSymbol
// Description	    : 
// Return type		: BOOL 
// Argument         : CString &name
// Argument         : Symbol *&simb
///////////////////////////////////////////////////////////////

BOOL CPascalCompiler::RetreaveSymbol(CString &name, Symbol *&simb)
{
	return m_SymbTableCollection.RetreaveSymbolAL (name,simb);
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:39:18 PM
// Function name	: CPascalCompiler::IncreaseLevel
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::IncreaseLevel()
{
	m_SymbTableCollection.IncreaseLevel ();	
	m_nVNivel++;
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:39:24 PM
// Function name	: CPascalCompiler::DecreaseLevel
// Description	    : 
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::DecreaseLevel()
{
	m_SymbTableCollection.DecreaseLevel ();	
	m_nVNivel--;
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 8/3/99 4:39:28 PM
// Function name	: SymbolTableCollection::RetreaveSymbolAL
// Description	    : 
// Return type		: BOOL 
// Argument         : CString &name
// Argument         : Symbol *&simb
///////////////////////////////////////////////////////////////

BOOL SymbolTableCollection::RetreaveSymbolAL(CString &name, Symbol *&simb)
{
	SymbolTable *ntable = NULL;
	POSITION pos = GetHeadPosition();
	while (pos != NULL)
	{
		ntable = GetNext(pos);
		if (ntable->Lookup(name,simb))
			return TRUE;
	}
	return FALSE;
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/20/99 4:26:32 PM
// Function name	: CPascalCompiler::GenerateCode
// Description	    : Generate a simple innstruction in the code table
// Return type		: void 
// Argument         : BYTE code
///////////////////////////////////////////////////////////////

void Code::GenerateCode(BYTE code)
{
	m_pbCodeTable[m_nCodeLength ++]	= code;
	if (m_bGenASM) 
		m_sASMCode +="\t\t" + CVirtualMachine::m_stInstNames [code] + "\n";
}





///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/25/99 1:21:26 PM
// Function name	: CPascalCompiler::GenerateCode
// Description	    : Generate a instruction with a register parameter
// Return type		: void 
// Argument         : BYTE code
// Argument         : REGISTER Param
///////////////////////////////////////////////////////////////

void Code::GenerateCode(BYTE code, REGISTER Param)
{
	m_pbCodeTable[m_nCodeLength ++] = code;
	*((REGISTER*)(&m_pbCodeTable[m_nCodeLength])) = Param;
	m_nCodeLength += VM_BYTES;
	if (m_bGenASM) 
	{
		CString sParam;
		sParam.Format(" %X \n",Param);
		m_sASMCode +="\t\t" + CVirtualMachine::m_stInstNames [code] + sParam;
	}
}

///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/25/99 1:21:26 PM
// Function name	: CPascalCompiler::GenerateCode
// Description	    : Generate a instruction with a register parameter
// Return type		: void 
// Argument         : BYTE code
// Argument         : REGISTER Param
///////////////////////////////////////////////////////////////

void Code::GenerateCode(BYTE code, REGISTER Param, CString &str)
{
	m_pbCodeTable[m_nCodeLength ++] = code;
	*((REGISTER*)(&m_pbCodeTable[m_nCodeLength])) = Param;
	m_nCodeLength += VM_BYTES;
	if (m_bGenASM) 
	{
		m_sASMCode += "\t\t" + CVirtualMachine::m_stInstNames [code] + " " +str+ "\n";
	}
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/25/99 1:23:54 PM
// Function name	: CPascalCompiler::GenerateCode
// Description	    : Generate a instruction with a byte parameter
// Return type		: void 
// Argument         : BYTE code
// Argument         : BYTE param
///////////////////////////////////////////////////////////////

void Code::GenerateCode(BYTE code, BYTE param)
{
	m_pbCodeTable[m_nCodeLength ++] = code;
	m_pbCodeTable[m_nCodeLength ++] = param;
	if (m_bGenASM) 
	{
		CString sParam;
		sParam.Format(" %X \n",param);
		m_sASMCode +="\t\t" + CVirtualMachine::m_stInstNames [code] + sParam;
	}
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/25/99 3:58:00 PM
// Function name	: CPascalCompiler::GenerateASMSource
// Description	    : Generate ASM source or not
// Return type		: void 
// Argument         : BOOL value
///////////////////////////////////////////////////////////////

void CPascalCompiler::GenerateASMSource(BOOL value)
{
	//m_bGenASM = value;
}




///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/25/99 4:45:35 PM
// Function name	: CPascalCompiler::GetASMCode
// Description	    : Returns a pointer to the ASm source code
// Return type		: CString * 
///////////////////////////////////////////////////////////////

CString * CPascalCompiler::GetASMCode()
{
	if (m_Code)
		return m_Code->GetASMCode();
	else return NULL;
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/25/99 5:31:26 PM
// Function name	: CPascalCompiler::Compile
// Description	    : COMPILE!!!!!!!!!!!!!!!1
// Return type		: void 
///////////////////////////////////////////////////////////////

void CPascalCompiler::Compile()
{
	delete m_Code;
	m_Code = new Code;
	ProgramSursa();
	GenerateCode(INSTR_STOP);
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/25/99 6:36:22 PM
// Function name	: CPascalCompiler::GenerateCode
// Description	    : Generates a instruction that uses a Stack Entry for specifiing
// Return type		: void 
// Argument         : BYTE code
// Argument         : StackEntry Param
///////////////////////////////////////////////////////////////

void Code::GenerateCode(BYTE code, StackEntry& Param)
{
	m_pbCodeTable[m_nCodeLength ++] = code;
	*((StackEntry*)(&m_pbCodeTable[m_nCodeLength])) = Param;
	m_nCodeLength += SE_SIZE;
	if (m_bGenASM) 
	{
		CString sParam;
		switch (Param.m_nType )
		{
		case SE_INTEGER:
			sParam.Format(" %d\n", CV_INTEGER(Param.m_Info));
			break;
		case SE_CHAR:
			sParam.Format(" %c\n", CV_CHAR(Param.m_Info));
			break;
		case SE_REAL:
			sParam.Format(" %d\n", CV_REAL(Param.m_Info));
			break;
		}
		m_sASMCode +="\t\t" + CVirtualMachine::m_stInstNames [code] + sParam;
	}
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/27/99 2:16:47 PM
// Function name	: CPascalCompiler::GenerateCode
// Description	    : This is used for generating the code for the Enter
// Return type		: void 
// Argument         : BYTE code
// Argument         : BYTE p1
// Argument         : REGISTER p2
///////////////////////////////////////////////////////////////

void Code::GenerateCode(BYTE code, BYTE p1, REGISTER p2)
{
	m_pbCodeTable[m_nCodeLength ++] = code;
	m_pbCodeTable[m_nCodeLength ++] = p1;
	*((REGISTER*)(&m_pbCodeTable[m_nCodeLength])) = p2;
	m_nCodeLength += VM_BYTES;
	if (m_bGenASM) 
	{
		CString sParam;
		sParam.Format(" %X, %X\n",p1,p2);
		m_sASMCode +="\t\t" + CVirtualMachine::m_stInstNames [code] + sParam;
	}
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/27/99 4:22:40 PM
// Function name	: CPascalCompiler::AddComment
// Description	    : Add a assembler comment
// Return type		: void 
// Argument         : CString str
///////////////////////////////////////////////////////////////
inline
void CPascalCompiler::AddComment(CString str)
{
	if (m_Code)
		m_Code->AddComment(str);
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 5/27/99 7:50:33 PM
// Function name	: CPascalCompiler::GenerateCode
// Description	    : parameter a signed stuff for relative addressing
// Return type		: void 
// Argument         : BYTE code
// Argument         : OFFSET offset
///////////////////////////////////////////////////////////////

void Code::GenerateCode(BYTE code,OFFSET Param)
{
	m_pbCodeTable[m_nCodeLength ++] = code;
	*((OFFSET*)(&m_pbCodeTable[m_nCodeLength])) = Param;
	m_nCodeLength += VM_BYTES;
	if (m_bGenASM) 
	{
		CString sParam;
		sParam.Format(" %d \n",Param);
		m_sASMCode +="\t\t" + CVirtualMachine::m_stInstNames [code] + sParam;
	}
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 6/2/99 10:32:44 AM
// Function name	: CPascalCompiler::StartRecord
// Description	    : Starts a code recording Session
// Return type		: POSITION - returns the position of the Record
///////////////////////////////////////////////////////////////

POSITION CPascalCompiler::StartNewRecord()
{
	Record *record = new Record;
	record->Start (m_Code);
	return m_RecordList.AddHead(record);
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 6/2/99 10:36:47 AM
// Function name	: CPascalCompiler::StopRecord
// Description	    : Stop the specified Recording
// Return type		: void 
// Argument         : POSITION pos
///////////////////////////////////////////////////////////////

void CPascalCompiler::StopRecord(POSITION pos)
{
	Record *record = (Record*) m_RecordList.GetAt(pos);
	record->Stop (m_Code);
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 6/2/99 10:41:55 AM
// Function name	: CPascalCompiler::RemoveRecord
// Description	    : Removes a recording
// Return type		: void 
// Argument         : POSITION pos
///////////////////////////////////////////////////////////////

void CPascalCompiler::RemoveRecord(POSITION pos)
{
	Record *record = (Record*) m_RecordList.GetAt(pos);
	delete record;
	m_RecordList.RemoveAt(pos);
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 6/2/99 10:45:47 AM
// Function name	: CPascalCompiler::PasteRecord
// Description	    : Put's the Recorded code into the code at the curent position
// Return type		: void 
// Argument         : POSITION pos
///////////////////////////////////////////////////////////////

void CPascalCompiler::PasteRecord(POSITION pos)
{
	Record *record = (Record*) m_RecordList.GetAt(pos);
	record->Paste (m_Code);	
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 6/3/99 10:48:36 AM
// Function name	: Code::GetASMCode
// Description	    : 
// Return type		: inline
///////////////////////////////////////////////////////////////

inline
CString* Code::GetASMCode()
{
	return &m_sASMCode;
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 6/3/99 10:48:49 AM
// Function name	: Code::AddComment
// Description	    : 
// Return type		: void 
// Argument         : CString str
///////////////////////////////////////////////////////////////

inline
void Code::AddComment(CString str)
{
	m_sASMCode += "; " + str + "\n";
}



///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 6/3/99 10:50:29 AM
// Function name	: CPascalCompiler::Redirect
// Description	    : This function redirects the code generation to the specified code object
// Return type		: Code * 
// Argument         : Code *code
///////////////////////////////////////////////////////////////

Code * CPascalCompiler::Redirect(Code *code)
{
	Code * result = m_Code;
	m_Code = code;
	return result;
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 6/3/99 11:56:16 AM
// Function name	: Code::SetAddress
// Description	    : Set's the address at the place specified in the code table
// Return type		: void 
// Argument         : int place
// Argument         : REGISTER value
///////////////////////////////////////////////////////////////
inline
void Code::SetAddress(int place, REGISTER value)
{
	*((REGISTER*) &m_pbCodeTable[place]) = value;
}


///////////////////////////////////////////////////////////////
// Programmer		: Zoly Farkas
// Creation Date	: 6/7/99 4:55:04 PM
// Function name	: Code::InsertLabel
// Description	    : Insert a Label into the code
// Return type		: void 
// Argument         : CString &str
///////////////////////////////////////////////////////////////

inline
void Code::InsertLabel(CString &str)
{
	m_sASMCode += str + "\n";
}
